clear all
clc
echo on
%
% This is the matlab file to construct H_2 and H_inf and h2/hinf
% solution of course example page 5-14, with second order
% nominal model. An iterative approach is used to refine the hinf
% controller design.
%             1
% Plant = ---------------
%        s^2 +0.2s + 1
%
%        10(s + 1)
% ws = -----------------
%     (s+0.01)(s + 10)
%
%      2*(s+10)^2
% wt = ------------
%       1000
%
% wu = 1
%
pause % strike any key to continue
%
% Construct the interconnected plant using sysic
%
s=zpk('s'); % zero=pole format
% plant transfer function
plant=1/(s^2+0.2*s+1);

% uncertainty weighting function
wt= (s+10)^2/500;
wt_1= 1/wt;

% augmented plant
Pw = plant*wt;

% performance weighting function
ws= 2*(s+1)/((s+0.01)*(s+10));

% Actuator effort weighting function
wu=tf(1,1);

% Now that the plant and the weight have been defined,
% construct the interconnection structure, P_IC;
echo off
systemnames = 'Pw wt_1 wu ws';
inputvar = '[yd; u]';
outputvar = '[Pw; ws ; wu; yd - wt_1]';
input_to_Pw = '[u]';
input_to_wt_1 ='[Pw]';
input_to_wu = '[u]';
input_to_ws = '[yd - wt_1]';
sysoutname = 'P_ic';
cleanupsysic = 'yes';
sysic
P_ic.InputName={'yd' 'u'}; % Set the input names
P_ic.OutputName={'Rob_st' 'Nom_p' 'Act_e' 'e'}; % Set the output names
P_ic=minreal(P_ic);

%clear plant Pw wt_1 wu ws
echo on
pause % strike any key to continue
clc
[k1,g1,gamma1,info1]=hinfsyn(P_ic,1,1,'Display','on','To',1e-4);
pause % strike any key to continue

clc
%
% Plot and analyze the sigma plot and objective function bode
%
echo off
omega=logspace(-2,3,200);
[mag_g1,phase_g1]=bode(g1,omega);
sig_g1=sigma(g1,omega);
semilogx(omega,sig_g1,omega,mag_g1(1,:),'--r', ...
omega,mag_g1(2,:),':g',omega,mag_g1(3,:),'-.m')
echo on

%
% As it is seen in the plot the actuator effort dominates the objective
% function. A better design would be to add frequency content to the
% actuator weigting function. Here we reduce the actuator effort at
% low frequency with a factor of 0.9 and permit higher amplitude at
% high frequencies, to obtain a better performance.
%
wu=0.9*(0.1*s+1)/(s+1);
pause % strike any key to continue
clc
% Regenerate the system interconnection P_ic
echo off
systemnames = 'Pw wt_1 wu ws';
inputvar = '[yd; u]';
outputvar = '[Pw; ws ; wu; yd - wt_1]';
input_to_Pw = '[u]';
input_to_wt_1 ='[Pw]';
input_to_wu = '[u]';
input_to_ws = '[yd - wt_1]';
sysoutname = 'P_ic';
cleanupsysic = 'yes';
sysic
P_ic.InputName={'yd' 'u'}; % Set the input names
P_ic.OutputName={'Rob_st' 'Nom_p' 'Act_e' 'e'}; % Set the output names
P_ic=minreal(P_ic);
echo on
pause % strike any key design the controller
clc
[k2,g2,gamma2,info2]=hinfsyn(P_ic,1,1,'Display','on','To',1e-4);

pause % strike any key to continue
clc
%
% Plot and analyze the sigma plot and objective function bode
%
echo off
[mag_g2,phase_g2]=bode(g2,omega);
sig_g2=sigma(g2,omega);
semilogx(omega,sig_g2,omega,mag_g2(1,:),'--r', ...
omega,mag_g2(2,:),':g',omega,mag_g2(3,:),'-.m')
echo on

%
% As it is seen in the plot by this change, the infinity norm of the
% performance index comes down to one.
%
% Look at the robust stability curve the dashed red line, it is
% way below one, so we expect good stability margin.
%
% Look at the nominal performance curve dotted-green. its peak is also
% beyond 0.6. Hence we can have better performance by increasing the
% bandwidth.
%
ws=2*(s+5)/((s+0.05)*(s+10));
echo off
systemnames = 'Pw wt_1 wu ws';
inputvar = '[yd; u]';
outputvar = '[Pw; ws ; wu; yd - wt_1]';
input_to_Pw = '[u]';
input_to_wt_1 ='[Pw]';
input_to_wu = '[u]';
input_to_ws = '[yd - wt_1]';
sysoutname = 'P_ic';
cleanupsysic = 'yes';
sysic
P_ic.InputName={'yd' 'u'}; % Set the input names
P_ic.OutputName={'Rob_st' 'Nom_p' 'Act_e' 'e'}; % Set the output names
P_ic=minreal(P_ic);
echo on
pause % strike any key design the controller
clc
[k3,g3,gamma3,info3]=hinfsyn(P_ic,1,1,'Display','on','To',1e-4);
pause % strike any key to continue

clc
%
% Plot and analyze the sigma plot and objective function bode
%
echo off
[mag_g3,phase_g3]=bode(g3,omega);
sig_g3=sigma(g3,omega);
semilogx(omega,sig_g3,omega,mag_g3(1,:),'--r', ...
omega,mag_g3(2,:),':g',omega,mag_g3(3,:),'-.m')
echo on

%
% Now Solve H2 solution using h2syn
%
[k4,g4,gamma4,info4]=h2syn(P_ic,1,1);
disp('The 2-norm of objective function is');
disp(gamma2);

%
% Plot and analyze the sigma plot and objective function bode
%
echo off
[mag_g4,phase_g4]=bode(g4,omega);
sig_g4=sigma(g4,omega);
semilogx(omega,sig_g4,omega,mag_g4(1,:),'--r', ...
omega,mag_g4(2,:),':g',omega,mag_g4(3,:),'-.m')
pause % strike a key to continue
clc
echo on

%%
% Design H2/Hinf controller: Nominal Performance (using Robust Toolbox)
%
% [K,CL,NORMZ,INFO] = H2HINFSYN(P,NMEAS,NCON,NZ2,WZ,key1,value1,...)
%
% Mixed H2/H-infinity synthesis with regional pole-placement
% constraints. NEW : ROBUST CONTROL TOOLBOX
%
% Given an LTI plant P with partitioned state-space form:
% dx/dt = A * x + B1 * w + B2 * u
% zinf = Ci * x + Di1 * w + Di2 * u
% z2 = C2 * x + D21 * w + D22 * u
% y = Cy * x + Dy1 * w + Dy2 * u
%
% H2HINFSYN employs LMI techniques to compute an output-feedback control
% law u = K(s)*y that
%
% * keeps the HINFNORM gain G from w to zinf below the value OBJ(1)
% * keeps the H2NORM H from w to z2 below the value OBJ(2)
% * minimizes a trade-off criterion of the form
%
% WEIGHT(1) * G^2 + WEIGHT(2) * H^2
% * places the closed-loop poles in the LMI region specified by REGION.
%
%
% Input:
%
% P LTI plant
% NMEAS Number of measurements (length of y)
% NCON Number of control channels (length of u)
% NZ2 length of z2
% WZ 1x2 vector of weights on zinf and z2 (i.e., weights for
% H-infinity and H2 performance, respectively)
%
% Option keywords and values:
% REGION Mx(2M) matrix [L,M] specifying the pole
% placement region as
% { z : L + z * M + conj(z) * M' < 0 }
% Use the interactive function LMIREG to generate
% REGION. The default REGION=[] enforces just closed-loop stability
%
% H2MAX upper bound on the H2 norm w->z2 (default = Inf)
% HINFMAX upper bound on the HINFNORM gain w->zinf (default = Inf)
% DKMAX bound on the norm of the feedthrough gain DK of K(s)
% (100=default, 0 yields a strictly proper controller)
% TOL desired relative accuracy on the objective value (default=1e-2)
% DISPLAY display synthesis information to screen, (default = 'off')
%
% Output:
%
% K optimal output-feedback controller
% CL lft(P,K) (closed-loop system)
% NORMZ 1x2 vector of closed-loop norms (HINFNORM gain w->zinf
% and H2 norm w->z2
%
% INFO struct array:
% INFO.R solution R of LMI solvability condition
% INFO.S solution S of LMI solvability condition

nmeas=1;
ncon =1;
nz2 =1;
h2max=1.2;
alpha=1;
beta =0;
wz=[alpha beta]; % alpha = 1; beta = 0 cost:= alpha* inf_norm + beta* 2-norm
[k5,g5,normz5,info5]=h2hinfsyn(ss(P_ic),nmeas,ncon,nz2,wz,...
'Display','off','h2max',h2max,'hinfmax',inf);

%
% Plot and analyze the sigma plot and objective function bode
%
echo off
[mag_g5,phase_g5]=bode(g5,omega);
sig_g5=sigma(g5(1:2,:),omega); % in this plot only
semilogx(omega,sig_g5,omega,mag_g5(1,:),'--r', ...
omega,mag_g5(2,:),':g',omega,mag_g5(3,:),'-.m')
pause %strike a key to continue
clc

% Plot a magnitude plot of the frequency response of k1 to k5
clf
bodemag(k1,'b',k2,'--r',k3,':g',k4,'-.m',k5,'c',omega)
legend('H_\infty_1','H_\infty_2','H_\infty_3','H_2','H_2/H_\infty')

